
import React from 'react';
import {
    TextInput,
} from 'react-native';


import {
    BaseViewComponent
} from 'wm-react-native-base';

/**
 * 封装Text
 */
export default class WMTextInput extends BaseViewComponent {

    // 构造
    constructor(props) {
        super(props);
        const { isShow } = this.props;
        this.state = {isShow: (isShow === undefined || isShow === null) ? true : isShow, cardExpiry: '', lastCardExpiry: ''};
    }

    _containChineseCharacter = (text) => {
        let reg = /[\u4E00-\u9FA5\uF900-\uFA2D]/;
        return reg.test(text);
    }

    /**
     * 纯数字
     * @param text
     * @returns {boolean}
     * @private
     */
    _isNumberCharacter = (text) => {
        let reg = /^[0-9]*$/;
        return reg.test(text);
    }

    /**
     * 数字
     * @param text
     * @returns {boolean}
     * @private
     */
    _isPunctuationNumberCharacter = (text) => {
        let reg = /^\d+(\.\d+)?$/;
        return reg.test(text);
    }

    /**
     *
     * @param text
     * @returns {boolean}
     * @private
     */
    _limitByKeyboard = (text)=>{
        //如果是数字键盘，只能输入数字
        let trimText = text.replace(/ /g, '');
        if(this.props.keyboardType){
            if(this.props.keyboardType === 'numeric' && !this._isNumberCharacter(trimText)){

                this.refs.input && this.refs.input.clear();

                setTimeout(()=>{
                    this.setState({
                        cardExpiry: ''
                    }, ()=>{
                        this.setState({
                            cardExpiry: this.state.lastCardExpiry
                        });
                    });
                })
                return true;
            }else if(this.props.keyboardType === 'numbers-and-punctuation'){
                if(trimText.indexOf('.') >=0){
                    if(trimText.indexOf('.') === trimText.lastIndexOf('.')){
                        if(!trimText.endsWith('.')){
                            if(!this._isPunctuationNumberCharacter(trimText)){
                                this.refs.input && this.refs.input.clear();

                                setTimeout(()=>{
                                    this.setState({
                                        cardExpiry: ''
                                    }, ()=>{
                                        this.setState({
                                            cardExpiry: this.state.lastCardExpiry
                                        });
                                    });
                                })
                                return true;
                            }
                        }
                    }else{
                        this.refs.input && this.refs.input.clear();

                        setTimeout(()=>{
                            this.setState({
                                cardExpiry: ''
                            }, ()=>{
                                this.setState({
                                    cardExpiry: this.state.lastCardExpiry
                                });
                            });
                        })
                        return true;
                    }
                }else{
                    if(!this._isPunctuationNumberCharacter(trimText)){
                        this.refs.input && this.refs.input.clear();

                        setTimeout(()=>{
                            this.setState({
                                cardExpiry: ''
                            }, ()=>{
                                this.setState({
                                    cardExpiry: this.state.lastCardExpiry
                                });
                            });
                        })
                        return true;
                    }
                }
            }
        }
    }

    _handlingCardExpiry = (text) => {

        //是否是正常的 TextInput 不处理任何逻辑
        const { isNormalInput } = this.props;
        if(isNormalInput){
            this._changeTxt(text);
            return ;
        }

        if(this._limitByKeyboard(text)){
            return ;
        }

        if(this.props.noneChinese && this._containChineseCharacter(text)){
            return ;
        }

        if (this.state.lastCardExpiry.length > text.length) {
            text = text.RTrim();
        } else {
            if (!this.props.isTelInput) {
                if (this.props.spaceLength >= 1 && text.length > 0 && text.length % (this.props.spaceLength + 1) === 0) {
                    // This is where the user has typed 2 numbers so far
                    // We can manually add a slash onto the end
                    // We check to make sure the current value was only 1 character
                    // long so that if they are backspacing, we don't add on the slash again
                    text = text.splice(text.length - 1, 0, ' ');
                }
            } else {
                if (text.length <= 4) {
                    if (text.length % 4 == 0) {
                        text = text.splice(text.length - 1, 0, ' ');
                    }
                } else {
                    let length = text.length - 4;
                    if (length > 0 && length % 5 == 0) {
                        text = text.splice(text.length - 1, 0, ' ');
                    }
                }
            }
        }

        // Update the state, which in turns updates the value in the text field
        this._changeTxt(text);
    }

    /**
     *
     * @param text
     * @private
     */
    _changeTxt = (text)=>{
        text = text+'';
        this.setState({
            lastCardExpiry: text,
            cardExpiry: text
        }, ()=>{
            this.props.onChangeTextInput && this.props.onChangeTextInput(text);
        });
    }

    getText = () => {
        //说明需要加入空格，则返回应该去掉空格
        if (this.props.isTelInput || this.props.spaceLength >= 1) {
            return this.state.cardExpiry.replace(/ /g, '');
        } else {
            if(this.props.keyboardType === 'numeric'){
                return String(Number(this.state.cardExpiry));
            }else if(this.props.keyboardType === 'numbers-and-punctuation'){
                return String(parseFloat(this.state.cardExpiry));
            }
            return this.state.cardExpiry;
        }
    }

    setText = (text) => {
        this.setState({
            cardExpiry: text
        });
    }

    getTrimText = () => {
        return this.state.cardExpiry.replace(/ /g, '');
    }

    setTelText = (value) => {
        if (value && typeof (value) === 'string' && value.length === 11) {
            let spaceTel = value.substring(0, 3) + " " + value.substring(3, 7) + " " + value.substring(7, 11);
            this._handlingCardExpiry(spaceTel);
        }
    }

    /**
     *
     * @param props
     */
    setNativeProps = (props)=>{
        this.refs.input && this.refs.input.setNativeProps(props);
    }

    /**
     * 获取焦点
     */
    focus = ()=>{
        this.refs.input && this.refs.input.focus();
    }

    /**
     * 清除焦点
     */
    clearFocus = ()=>{
        this.refs.input && this.refs.input.blur && this.refs.input && this.refs.input.blur();
    }

    /**
     * 是否有焦点
     * @returns {*|boolean}
     */
    isFocused = ()=>{
        return this.refs.input && this.refs.input.isFocused();
    }

    /**
     * 当前是否已经显示
     */
    isShow = ()=>{
        return this.state.isShow;
    }

    /**
     * 隐藏
     */
    hidden = ()=>{
        this.setState({isShow:false});
    }

    /**
     * 显示
     */
    show = ()=>{
        this.setState({isShow:true});
    }

    render() {
        if(!this.state.isShow){
            return null;
        }

        let { keyboardType } = this.props;
        if(keyboardType === 'numbers-and-punctuation'){
            keyboardType = 'numeric';
        }

        return (
            <TextInput
                spaceLength={0}
                onChangeText={this._handlingCardExpiry}
                placeholderTextColor='#CBCBCB'
                {...this.props}
                // value={this.state.cardExpiry}
                defaultValue={this.state.cardExpiry}
                underlineColorAndroid='transparent'
                keyboardType={keyboardType}
                ref='input'
            />
        );
    }
}