/**
 * Created with IntelliJ IDEA.
 * User: aaronlin
 * Date: 2018/6/2
 * Time: 15:31
 */

import React,{ PureComponent } from 'react';
import ReactNative, {
    View,
    Text,
    Animated,
    PanResponder,
    Platform,
    UIManager,
} from 'react-native';

import {
    StyleSheetPropType,
    ViewStylePropTypes,
    EventEmitter,
} from '../RNExt';

const viewStylePropType = StyleSheetPropType(ViewStylePropTypes);


import Refreshing from '../refreshlistview/Refreshing';

/**
 *
 */
const AnimatedRefreshingView = Animated.createAnimatedComponent(Refreshing);

/**
 * 动画时间
 * @type {number}
 */
const ANIM_DURATION = 100;

/**
 *
 * @type {number}
 */
const MAX_REFRESH_DISTANCE = 160;

/**
 *
 * @type {number}
 */
const REFRESH_HEIGHT = 50;

/**
 *
 * @type {string}
 */
const REFRESH_EVENT = 'onRefresh';
/**
 *
 * @type {string}
 */
const END_REFRESH_EVENT = 'onEndRefresh';
/**
 *
 * @type {string}
 */
const BEGIN_REFRESH_EVENT = 'beginRefresh';

export default class PullToRefreshContainerV2 extends PureComponent{

    static propTypes = {
        headerStyle:viewStylePropType,  //刷新头 style
    }

    /**
     * 发送下拉刷新结束事件
     * @param pullRefreshEmitter
     */
    static sendEndRefreshEvent(pullRefreshEmitter : EventEmitter) : void{
        pullRefreshEmitter && pullRefreshEmitter.emit(END_REFRESH_EVENT);
    }

    /**
     * 发送开始下拉刷新事件
     * @param pullRefreshEmitter
     */
    static sendBeginRefreshEvent(pullRefreshEmitter : EventEmitter) : void{
        pullRefreshEmitter && pullRefreshEmitter.emit(BEGIN_REFRESH_EVENT);
    }

    /**
     * 监听下拉刷新事件
     * @param pullRefreshEmitter
     */
    static addRefreshListener(pullRefreshEmitter : EventEmitter, refreshCallback : Function) : void{
        refreshCallback && pullRefreshEmitter && pullRefreshEmitter.addListener(REFRESH_EVENT, refreshCallback);
    }

    /**
     * 删除下拉刷新事件
     * @param pullRefreshEmitter
     */
    static removeRefreshListener(pullRefreshEmitter : EventEmitter, refreshCallback : Function) : void{
        refreshCallback && pullRefreshEmitter && pullRefreshEmitter.removeListener(REFRESH_EVENT, refreshCallback);
    }

    /**
     *
     * @type {null}
     * @private
     */
    _panResponder = null;

    /**
     * 是否正在动画
     * @type {boolean}
     * @private
     */
    _isAniming = false;

    /**
     * 是否正在下拉刷新
     * @type {boolean}
     * @private
     */
    _isRefreshing = false;

    /**
     *
     * @type {boolean}
     * @private
     */
    _isReadyForPullStart = false;

    /**
     * Event
     */
    _pullRefreshEmitter;

    constructor(props) {
        super(props);

        this._initRefreshEmitter();
        this._initPanResponder();

        this.state = {
            headerRefreshTranslateYAnim: new Animated.Value(0),
            headerRefreshOpacityAnim: new Animated.Value(0),
            contentTranslateYAnim: new Animated.Value(0),
        }
    }

    componentWillUnmount() {
        this._pullRefreshEmitter.removeAllListeners();
    }

    /**
     * 初始化刷新事件
     * @private
     */
    _initRefreshEmitter = ()=>{
        this._pullRefreshEmitter = new EventEmitter();
        this._pullRefreshEmitter.addListener(END_REFRESH_EVENT, this.endRefreshing);
        this._pullRefreshEmitter.addListener(BEGIN_REFRESH_EVENT, this.beginRefreshing);
    }

    /**
     *
     * @private
     */
    _initPanResponder = ()=>{
        /**
         *
         * @param evt
         * @param gestureState
         * @private
         */
        const _onStartShouldSetPanResponder = (evt, gestureState)=>{
            // console.log('onStartShouldSetPanResponder===========> ', gestureState.dy);
            return this._isReadyForPullStart;
        };
        /**
         * move拦截
         * @param evt
         * @param gestureState
         * @returns {boolean}
         * @private
         */
        const _onMoveShouldSetPanResponder = (evt, gestureState)=>{
            // console.log('onMoveShouldSetPanResponder===========> ', gestureState.dy);
            const { dx, dy } = gestureState;

            if(this._isReadyForPullStart){
                return true;
            }
            if(this._isAniming || this._isRefreshing){
                return false;
            }

            const { isReadyForPullStart } = this.props;
            if(isReadyForPullStart){
                this._isReadyForPullStart = !!isReadyForPullStart(evt, gestureState);
                // console.log('_isReadyForPullStart=======> ', this._isReadyForPullStart);
                return this._isReadyForPullStart;
            }

            return false;
        };

        /**
         * move
         * @param evt
         * @param gestureState
         * @private
         */
        const _onPanResponderMove = (evt, gestureState) => {
            const { dx, dy } = gestureState;
            // console.log('onPanResponderMove================> dy: ', dy);
            if(dy >=MAX_REFRESH_DISTANCE/5 && dy <= MAX_REFRESH_DISTANCE){
                if(dy >= MAX_REFRESH_DISTANCE / 4){
                    this.state.headerRefreshOpacityAnim.setValue(1);
                }
                this.state.headerRefreshTranslateYAnim.setValue(dy);
                this.state.contentTranslateYAnim.setValue(dy);
            }
        };

        /**
         * 释放
         * @private
         */
        const _onPanResponderRelease = (evt, gestureState)=>{
            const { dx, dy } = gestureState;
            // console.log('onPanResponderRelease================> dy: ', dy);

            this._isAniming = true;

            if(dy <= MAX_REFRESH_DISTANCE / 1.8){  //恢复
                this._restoreRefreshAnim();
            }else{  //下拉刷新开始
                this._startRefreshAnim();
            }
            this._isReadyForPullStart = false;
        };

        this._panResponder = PanResponder.create({
            // 要求成为响应者：
            onStartShouldSetPanResponder: _onStartShouldSetPanResponder,
            onStartShouldSetPanResponderCapture: _onStartShouldSetPanResponder,
            onMoveShouldSetPanResponder: _onMoveShouldSetPanResponder,
            onMoveShouldSetPanResponderCapture: _onMoveShouldSetPanResponder,
            /**
             * 开始手势操作。给用户一些视觉反馈，让他们知道发生了什么事情！
             * gestureState.{x,y} 现在会被设置为0
             * @param evt
             * @param gestureState
             */
            onPanResponderGrant: (evt, gestureState) => {
                const { dx, dy } = gestureState;
                // console.log('onPanResponderGrant================> dy: ', dy);
            },
            /**
             * 最近一次的移动距离为gestureState.move{X,Y}
             * 从成为响应者开始时的累计手势移动距离为gestureState.d{x,y}
             * @param evt
             * @param gestureState
             */
            onPanResponderMove: _onPanResponderMove,
            /**
             * 用户放开了所有的触摸点，且此时视图已经成为了响应者。
             * 一般来说这意味着一个手势操作已经成功完成。
             */
            onPanResponderRelease: _onPanResponderRelease,
            /**
             * 另一个组件已经成为了新的响应者，所以当前手势将被取消。
             */
            onPanResponderTerminate: _onPanResponderRelease,
            /**
             * 监视器被要求终止，返回false表示不同意终止，或者不处理
             * @param evt
             * @param gestureState
             * @returns {boolean}
             */
            onPanResponderTerminationRequest: (evt, gestureState) => {
                // console.log('onPanResponderTerminationRequest=======>')
                _onPanResponderRelease(evt, gestureState);
                return false;
            },
            /**
             * 返回一个布尔值，决定当前组件是否应该阻止原生组件成为JS响应者
             * 默认返回true。目前暂时只支持android。
             * @param evt
             * @param gestureState
             * @returns {boolean}
             */
            onShouldBlockNativeResponder: (evt, gestureState) => {
                // console.log('onShouldBlockNativeResponder=======>')
                return true;
            },
        });
    }

    /**
     * 恢复刷新动画
     * @param isEndRefreshing 是否结束刷新
     * @private
     */
    _restoreRefreshAnim = (isEndRefreshing : Boolean = false)=>{
        Animated.parallel([
            Animated.timing(this.state.headerRefreshOpacityAnim, {
                toValue: 0,
                duration: ANIM_DURATION,
                isInteraction: false,
                useNativeDriver: true,
            }),
            Animated.timing(this.state.headerRefreshTranslateYAnim, {
                toValue: 0,
                duration: ANIM_DURATION,
                isInteraction: false,
                useNativeDriver: true,
            }),
            Animated.timing(this.state.contentTranslateYAnim, {
                toValue: 0,
                duration: ANIM_DURATION,
                isInteraction: false,
                useNativeDriver: true,
            })
        ]).start(()=>{
            this.refs.headerRefreshView && this.refs.headerRefreshView._component.stopAnim();
            this._isAniming = false;
            isEndRefreshing && (this._isRefreshing = false);
        });
    }

    /**
     * 开始下拉刷新动画
     * @private
     */
    _startRefreshAnim = ()=>{
        this._isRefreshing = true;
        Animated.parallel([
            Animated.timing(this.state.headerRefreshTranslateYAnim, {
                toValue: MAX_REFRESH_DISTANCE / 2,
                duration: ANIM_DURATION,
                isInteraction: false,
                useNativeDriver: true,
            }),
            Animated.timing(this.state.contentTranslateYAnim, {
                toValue: MAX_REFRESH_DISTANCE / 2,
                duration: ANIM_DURATION,
                isInteraction: false,
                useNativeDriver: true,
            })
        ]).start(()=>{
            this.refs.headerRefreshView && this.refs.headerRefreshView._component.startAnim();
            this._isAniming = false;
            this._notificationRefreshCallback();
        });
    }

    /**
     * 通知刷新开始 回调
     * @private
     */
    _notificationRefreshCallback = ()=>{
        const { onRefresh } = this.props;
        onRefresh && onRefresh();

        this._pullRefreshEmitter.emit(REFRESH_EVENT);
    }

    /**
     * 自动下拉刷新
     */
    beginRefreshing = ()=>{
        if (!this._isRefreshing) {
            this.state.headerRefreshOpacityAnim.setValue(1);
            this._startRefreshAnim();
        }
    }

    /**
     * 刷新结束
     */
    endRefreshing = () => {
        if (this._isRefreshing) {
            this._restoreRefreshAnim(true);
        }
    }

    /**
     * 头部 刷新 view
     * @returns {XML}
     * @private
     */
    _renderHeaderRefreshView = ()=>{

        const { headerStyle } = this.props;

        return (
            <AnimatedRefreshingView
                ref={'headerRefreshView'}
                style={[
                    {height:REFRESH_HEIGHT, position:'absolute', top:-((MAX_REFRESH_DISTANCE / 2 - REFRESH_HEIGHT) / 2 + REFRESH_HEIGHT), left:0, right:0, zIndex:5},
                    {opacity:this.state.headerRefreshOpacityAnim},
                    {transform: [{translateY: this.state.headerRefreshTranslateYAnim}]},
                    headerStyle,
                ]}
                isShow={true}
                text={'下拉刷新'}
            />
        );
    }

    /**
     *
     * @private
     */
    _onLayout = (e)=>{
        const { onLayout } = this.props;
        onLayout && onLayout(e);
    }

    render(){
        const { children, style } = this.props;

        return (
            <View
                onLayout={this._onLayout}
                testID={'pullToRefreshContainerViewTestID'}
                ref={'pullToRefreshContainerView'}
                {...this._panResponder.panHandlers}
                style={[styles.container, style]}
                pointerEvents={'box-none'}
            >

                { this._renderHeaderRefreshView() }

                <Animated.View
                    ref={'contentView'}
                    pointerEvents={'box-none'}
                    style={[{flex:1}, {transform:[{translateY:this.state.contentTranslateYAnim}]}]}
                >
                    { children }
                </Animated.View>

            </View>
        );
    }

}

const styles = {
    container:{
        flex:1,
    },
};